#ifndef _EXAMPLE_H_
#define _EXAMPLE_H_

#define SEL_MENU_MODE            2
#define SEL_TITLE_MODE           4
#define SEL_ADC_LPM35_MODE       6
#define SEL_DISPLAY_MODE         8
#define SEL_BAT_LOW_MODE        10
#define SEL_UART_TRANSFER_MODE  12

extern char takeADCReading;

// Function prototypes
extern void ULP_meter(void);

/*Energy consumption task*/
void vTask_EnergyTask(void);
void vTask_NormalTask(void);
void vTask_AES(void);
void vTask_CRC(void);
void vTask_FFT(void);

void vEnergyTask_AES(void);
void vEnergyTask_CRC(void);
void vEnergyTask_FFT(void);

#endif
